#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
 
#include <netdb.h>
#include <net/if.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <ctype.h>

#include "net_op.h"

// function declare
int GetLocalMac(const char *eth_inf, char *mac); // 获取本机mac
int GetLocalIp(const char *eth_inf, char *ip); // 获取本机ip
int GetLocalNetMask(const char *eth_inf,char* netmask_addr); //获取本机mask
int GetLocalGateWay(char* gateway);//获取本机gateway 
 

/****** main test **********/
int get_net(NET_WORK_ATTR *net_attr)
{
	// NET_WORK_ATTR net_attr = {0};
	// sprintf((*net_attr).name, eth);

    GetLocalMac((*net_attr).name, (*net_attr).mac);
    // printf("local %s mac: %s\n", (*net_attr).name, (*net_attr).mac);
    
    GetLocalIp((*net_attr).name, (*net_attr).ip);
    // printf("local %s ip: %s\n", (*net_attr).name, (*net_attr).ip);
    
    GetLocalNetMask((*net_attr).name,(*net_attr).netmask_addr);
    // printf("local %s netmask:%s\n",(*net_attr).name,(*net_attr).netmask_addr); 
    
    GetLocalGateWay((*net_attr).gateway);	
    // printf("local %s gateway:%s\n",(*net_attr).name,(*net_attr).gateway);  

	return 0;
}
 
 
// 获取本机mac
int GetLocalMac(const char *eth_inf, char *mac)
{
	struct ifreq ifr;
	int sd;
	
	bzero(&ifr, sizeof(struct ifreq));
	if( (sd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
		printf("get %s mac address socket creat error\n", eth_inf);
		return -1;
	}
	
	strncpy(ifr.ifr_name, eth_inf, sizeof(ifr.ifr_name) - 1);
 
	if(ioctl(sd, SIOCGIFHWADDR, &ifr) < 0)
	{
		printf("get %s mac address error\n", eth_inf);
		close(sd);
		return -1;
	}
 
	snprintf(mac, MAC_SIZE, "%02x:%02x:%02x:%02x:%02x:%02x",
		(unsigned char)ifr.ifr_hwaddr.sa_data[0], 
		(unsigned char)ifr.ifr_hwaddr.sa_data[1],
		(unsigned char)ifr.ifr_hwaddr.sa_data[2], 
		(unsigned char)ifr.ifr_hwaddr.sa_data[3],
		(unsigned char)ifr.ifr_hwaddr.sa_data[4],
		(unsigned char)ifr.ifr_hwaddr.sa_data[5]);
 
	close(sd);
	
	return 0;
}
 
 
 
// 获取本机ip
int GetLocalIp(const char *eth_inf, char *ip)
{
	int sd;
	struct sockaddr_in sin;
	struct ifreq ifr;
 
	sd = socket(AF_INET, SOCK_DGRAM, 0);
	if (-1 == sd)
	{
		printf("socket error: %s\n", strerror(errno));
		return -1;		
	}
 
	strncpy(ifr.ifr_name, eth_inf, IFNAMSIZ);
	ifr.ifr_name[IFNAMSIZ - 1] = 0;
	
	// if error: No such device
	if (ioctl(sd, SIOCGIFADDR, &ifr) < 0)
	{
		printf("ioctl error: %s\n", strerror(errno));
		close(sd);
		return -1;
	}
 
	memcpy(&sin, &ifr.ifr_addr, sizeof(sin));
	snprintf(ip, IP_SIZE, "%s", inet_ntoa(sin.sin_addr));
 
	close(sd);
	return 0;
}

//获取本机mask
int GetLocalNetMask(const char *eth_inf,char* netmask_addr)  
{  
    int sock_netmask;   
    struct ifreq ifr_mask;  
    struct sockaddr_in *net_mask;  
          
    sock_netmask = socket( AF_INET, SOCK_STREAM, 0 );  
    if( sock_netmask == -1)  
    {  
        perror("create socket failture...GetLocalNetMask\n");  
        return -1;  
    }  
      
    memset(&ifr_mask, 0, sizeof(ifr_mask));     
    strncpy(ifr_mask.ifr_name, eth_inf, sizeof(ifr_mask.ifr_name )-1);     
  
    if( (ioctl( sock_netmask, SIOCGIFNETMASK, &ifr_mask ) ) < 0 )   
    {  
        printf("mac ioctl error\n");  
        return -1;  
    }  
      
    net_mask = ( struct sockaddr_in * )&( ifr_mask.ifr_netmask );  
    strcpy( netmask_addr, inet_ntoa( net_mask -> sin_addr ) );             
    close( sock_netmask );  
    return 0;  
}
//获取本机gateway
int GetLocalGateWay(char* gateway)  
{  
    FILE *fp;  
    char buf[512];  
    char cmd[128];   
    char *tmp;  
  
    strcpy(cmd, "ip route");  
    fp = popen(cmd, "r");  
    if(NULL == fp)  
    {  
        perror("popen error");  
        return -1;  
    }  
    while(fgets(buf, sizeof(buf), fp) != NULL)  
    {  
        tmp =buf;  
        while(*tmp && isspace(*tmp))  
            ++ tmp;  
        if(strncmp(tmp, "default", strlen("default")) == 0)  
            break;  
    }  
    sscanf(buf, "%*s%*s%s", gateway);          
    pclose(fp);  
      
    return 0;  
}
