/*
 ============================================================================
 Name        : test.c
 Author      : LiYongjun
 Version     : 1.0
 Copyright   : Your copyright notice
 Description : Hello World in C, Ansi-style
 ============================================================================
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <errno.h>
#include "net_op.h"
#include "eeprom_flash_map.h"
#include "cJSON.h"

#define RET_OK 0
#define RET_ERR -1

#define LISTEN_QUEUE_NUM 5

#define BUFFER_SIZE 1024

#define ECHO_PORT 39555

static int ExecuteCMD(unsigned char *cmd, unsigned char *result, int size);

int main(int argc, char *argv[])
{
    int debug_flag = 0;
    if (argc == 2)
        if (strcmp(argv[1], "-d") == 0)
            debug_flag = 1;

    char ver[20] = "1.0.0.3";

    if (debug_flag == 1)
        printf("ver : %s\n%s %s\n", ver, __DATE__, __TIME__);
    fflush(stdout);

    int sockfd, opt = 1;
    uint32_t len;
    struct sockaddr_in cliaddr;
    struct sockaddr_in sevaddr;
    uint8_t buffer[BUFFER_SIZE];
    int ret = RET_OK;

    if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
        if (debug_flag == 1)
            perror("ERROR opening socket");
        return RET_ERR;
    }

    //SO_REUSEADDR允许重用本地址/*不会卡住1分钟*/
    if ((ret = setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt))) < 0)
    {
        if (debug_flag == 1)
            perror("ERROR setsockopt");
        goto failed;
    }

    ret = setsockopt(sockfd, SOL_SOCKET, SO_BROADCAST, (char *)&opt, sizeof(opt));
    if (ret == -1)
        if (debug_flag == 1)
            printf("Set sock to broadcast format fail\n");

    memset(&sevaddr, 0, sizeof(sevaddr));
    sevaddr.sin_family = AF_INET;
    sevaddr.sin_addr.s_addr = htonl(INADDR_ANY);
    sevaddr.sin_port = htons(ECHO_PORT);

    if ((ret = bind(sockfd, (struct sockaddr *)&sevaddr, sizeof(sevaddr))) < 0)
    {
        if (debug_flag == 1)
            perror("ERROR on binding");
        goto failed;
    }

    do
    {
        len = sizeof(cliaddr);
        if ((ret = recvfrom(sockfd, buffer, sizeof(buffer) - 1, 0, (struct sockaddr *)&cliaddr, &len)) > 0)
        {
            buffer[ret] = 0;
            if (debug_flag == 1)
            {
                printf("recv:[%s:%d]%s\n", inet_ntoa(cliaddr.sin_addr), ntohs(cliaddr.sin_port), buffer);
                printf("len = %d\n", ret);
                int i;
                for (i = 0; i < ret; i++)
                    printf("0x%02x ", buffer[i]);
                printf("\n");
                fflush(stdout);
            }
            if (strcmp("Anosi_search", buffer) == 0)
            {

                char result[50] = {0};
                ExecuteCMD("iptool --dump_wan_eth_port", result, 49);
                if (debug_flag == 1)
                    printf("%s", result);
                int result_len = strlen(result);
                if (result[result_len - 1] == '\n')
                    result[result_len - 1] = '\0';
                // printf("%d\n", strcmp("eth1", result));

                NET_WORK_ATTR net_attr = {0};

                strcpy(net_attr.name, result);

                get_net(&net_attr);
                if (debug_flag == 1)
                {
                    printf("%s mac: %s\n", (net_attr).name, (net_attr).mac);
                    printf("%s ip: %s\n", (net_attr).name, (net_attr).ip);
                    printf("%s netmask:%s\n", (net_attr).name, (net_attr).netmask_addr);
                    printf("%s gateway:%s\n", (net_attr).name, (net_attr).gateway);
                }

                char serialNo[32] = {0};
                // eeprom_get_product_sn(serialNo);
                am335x_eeprom_dump(PRODUCT_SN_BASE, PRODUCT_SN_MAX_LENGTH, serialNo);
                if (debug_flag == 1)
                    printf("sn = %s\n", serialNo);
                char name[32] = {0};
                // eeprom_get_produce_name(name);
                // if (debug_flag == 1)
                //     printf("name = %s\n", name);
                FILE *fp;
                fp = fopen("/etc/hostname", "r");
                // char name[100] = {0};
                if (fgets(name, sizeof(name) - 1, fp) != NULL) /*读取*/
                {
                    if (debug_flag == 1)
                        printf("read %s", name);
                    int length = strlen(name);
                    if (name[length - 1] == '\n')
                    {
                        name[length - 1] = '\0';
                    }
                }
                fclose(fp); /*关闭*/
                if (debug_flag == 1)
                    printf("name = %s\n", name);
                cJSON *root = cJSON_CreateObject();
                cJSON_AddStringToObject(root, "SN", serialNo);
                cJSON_AddStringToObject(root, "IP", net_attr.ip);
                cJSON_AddStringToObject(root, "MAC", net_attr.mac);
                cJSON_AddStringToObject(root, "NA", name);
                cJSON_AddStringToObject(root, "VER", ver);
                char *out = cJSON_Print(root);

                memset(&cliaddr, 0, sizeof(cliaddr));
                cliaddr.sin_family = AF_INET;
                cliaddr.sin_addr.s_addr = htonl(INADDR_BROADCAST);
                cliaddr.sin_port = htons(ECHO_PORT + 1);
                ret = sendto(sockfd, out, strlen(out), 0, (struct sockaddr *)&cliaddr, len);
                if (debug_flag == 1)
                    printf("ret = %d\n", ret);
                if (debug_flag == 1)
                    perror("");

                fflush(stdout);
                free(out);
                out = NULL;
                cJSON_Delete(root);
                root = NULL;
            }
            else
            {
                if (debug_flag == 1)
                    printf("message error\n");
            }
        }
        else
        {
            if (debug_flag == 1)
                printf("Recv msg err\n");
        }
    } while (ret >= 0);

failed:
    printf("aaa\n");
    close(sockfd);
    return 0;
}

/**
 * @brief get cmd resuslts
 *
 * @param [in] unsigned char* cmd
 * @param [in] unsigned char* result
 * @param [in] int size
 * @param [out] None
 *
 * @return
 *
 * @history
 * 1.Date         : 2018-11-26 15:20:27
 *   Author       : panzidong
 *   Modification : Created function
 */
static int ExecuteCMD(unsigned char *cmd, unsigned char *result, int size)
{
    FILE *fp = popen(cmd, "r");
    if (NULL == fp)
    {
        printf("popen error\n");
        return -1;
    }

    int nbytes = fread(result, sizeof(unsigned char), size, fp);
    if (nbytes > 0 && nbytes < size - 1)
        result[nbytes] = '\0';

    pclose(fp);
    return nbytes;
}